@@ -1,3 +1,4 @@
+from platform import system
 from subprocess import Popen, PIPE
 from typing import Tuple, List, Union
 import re
@@ -34,7 +35,8 @@ def execute_command(command: Union[List[str], str], input_text: str = '', cwd: s
     :return: A tuple with the standard and error outputs.
     """
     command = split_arg_string(command) if isinstance(command, str) else command
-    process = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE, universal_newlines=True, cwd=cwd)
+    shell = system() == 'Windows'
+    process = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE, universal_newlines=True, cwd=cwd, shell=shell)
 
     process.stdin.write(f'{input_text}\n')
 
