@@ -13,6 +13,7 @@ from sys import stdout
 from shutil import move
 from typing import Union, Optional, TextIO, Any, List, Tuple, IO
 import glob
+from string import ascii_letters, digits
 
 
 def expand_wildcards(*filenames: Union[PathLike, str, bytes]) -> List[str]:
@@ -246,10 +247,14 @@ def remove_files(*files: Union[PathLike, str, bytes], ignore_errors: bool = Fals
     """
     for file in expand_wildcards(*files):
         if isdir(file):
-            if recursive:
-                rmtree(file)
-            else:
-                rmdir(file)
+            try:
+                if recursive:
+                    rmtree(file)
+                else:
+                    rmdir(file)
+            except Exception as e:
+                if not ignore_errors:
+                    raise e
         elif not ignore_errors or exists(file):
             remove(file)
 
@@ -437,6 +442,7 @@ def read_from(filename: Union[PathLike, str, bytes],
     """
     pattern = re.compile(regex, flags=re.IGNORECASE if ignore_case else 0)
     from_detected, lines = not regex, []
+
     with open_file(filename, 'rt') as file:
         for line in file:
             from_detected = from_detected or pattern.match(line)
@@ -457,6 +463,7 @@ def read_until(filename: Union[PathLike, str, bytes], regexp: str = '',
     """
     pattern = re.compile(regexp, flags=re.IGNORECASE if ignore_case else 0)
     until_detected, lines = False, []
+
     with open_file(filename, 'rt') as file:
         for line in file:
             until_detected = until_detected or regexp and pattern.match(line)
@@ -465,6 +472,50 @@ def read_until(filename: Union[PathLike, str, bytes], regexp: str = '',
     return lines
 
 
+def read_body(filename: Union[PathLike, str, bytes], from_re: str = '', until_re: str = '',
+               ignore_case: bool = False, line_break: bool = True) -> List[str]:
+    """ Read from the line that matches with a from_re regular expression until the line that matches with until_re.
+
+    :param filename: The path to the file.
+    :param from_re: The regular expression.
+    :param from_re: The regular expression.
+    :param ignore_case: If ignore case or not.
+    :param line_break: If True, the newline character is conserved, otherwise is removed.
+    :return: A list of strings with each file line.
+    """
+    from_pattern = re.compile(from_re, flags=re.IGNORECASE if ignore_case else 0)
+    until_pattern = re.compile(until_re, flags=re.IGNORECASE if ignore_case else 0)
+    from_detected, until_detected, lines = not from_re, False, []
+
+    with open_file(filename, 'rt') as file:
+        for line in file:
+            from_detected = from_detected or from_pattern.match(line)
+            until_detected = until_detected or until_re and until_pattern.match(line)
+
+            if from_detected and not until_detected:
+                lines.append(line[:-1] if not line_break and line[-1] == '\n' else line)
+
+        return lines
+
+
+def read_line(filename: Union[PathLike, str, bytes], regexp: str,
+              ignore_case: bool = False, line_break: bool = True) -> Optional[str]:
+    """ Read the line that matches with a regular expression.
+
+    :param filename: The path to the file.
+    :param regexp: The regular expression.
+    :param ignore_case: If ignore case or not.
+    :param line_break: If True, the newline character is conserved, otherwise is removed.
+    :return: A string with the line or None.
+    """
+    pattern = re.compile(regexp, flags=re.IGNORECASE if ignore_case else 0)
+    with open_file(filename, 'rt') as file:
+        for line in file:
+            if pattern.match(line):
+                return line[:-1] if not line_break and line[-1] == '\n' else line
+    return None
+
+
 def write_file(filename: Union[PathLike, str, bytes, int], content: Union[str, List[str]]) -> None:
     """ Write a file.
 
@@ -564,3 +615,13 @@ def has_encoding(file: Union[PathLike, str, bytes], encoding: str) -> bool:
     except UnicodeDecodeError:
         return False
     return True
+
+
+def to_filename(text: str, ext: str = '') -> str:
+    """ Convert a string to a valid filename.
+    :param text: The string to convert.
+    :param ext: The file extension.
+    :return: The converted string.
+    """
+    valid_chars = f'-_.() {ascii_letters}{digits}'
+    return ''.join(c if c in valid_chars else '_' for c in text) + ext
